/*!
CLogger.cpp
 Menu Disc Demo
 
 Author: Jeroen Verbeek
 Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
 
 This software is provided without any guarantee.
 No responsibility taken for any problems caused by the use if this software.
 
 Anyone is authorised to use this program code and methods without warranty,
 all that is required is that you contact me (jeroen@happymakinggames.com),
 and tell me that you use it and what you use it in -
 all I want is to know is who uses it and where.
 
 I will not ask for payment in any way: this code is free.
 If you encounter a bug with the code, please tell me about it.
 If you have a bug-fix for the code please provide me with it.
 If you think of an improvement please tell me about it.
 You are not required to provide me with bug reports, bug fixes or improvements.
 If you require me to perform any work related to this code or
 not related to this code, please contact me: I am always open to offers.
 
 Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
 specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
 I am a big fan of circular menus (disc menus).
 I am of the opinion that they provide the quickest method of selecting an action
 from a structured tree of actions. A menu of actions organised following this method
 will, in time, allow almost "blind selection" of actions. Each action selection
 requires the same distance of mouse movement, just in a slightly diferent angle.
 In my opinion this seems to work best with menu discs of no more than 8 items.
 There is plenty of research to be found through the internet regarding the 
 advantages of circular menus, for example: muscle memory.
 Some websites I recommend you look at are:
 
 http://catalog.com/hopkins/piemenus/piemenus-hopkins.html
 
 http://www.piemenus.com/
 
 http://www.sm.luth.se/~david/carl/www/argonaut.html
 
 Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
 */
/*-----------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "CLogger.h"

/*-----------------------------------------------------------------------------------------------*/
/*!
\var CLogger* g_pGlobalLog = 0;
 
Global log object. Needs to be created before use, CLogger is not a singleton class.
 */
/*-----------------------------------------------------------------------------------------------*/
CLogger* g_pGlobalLog = 0;


/*-----------------------------------------------------------------------------------------------*/
/*!
Global function to check if the <b>GLOBAL LOG FILE</b> exists. 

\return True if the global log file has been created, or false.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CLoggerGlobalLogExists(void)
{
	if(g_pGlobalLog != 0)
		return true;
	return false;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Consturctor. Opens/creates the logfile, writes the header.
 
\param		pcLogFileName Filename for the logfile.
 */
/*-----------------------------------------------------------------------------------------------*/
CLogger::CLogger(char* pcLogFileName)
{
	m_pcLogFileName = new char [ strlen(pcLogFileName) + 1 ];
	strcpy(m_pcLogFileName, pcLogFileName);
	m_hLogFile = fopen(m_pcLogFileName, "w");
	LogStart();
	LogMessage("Log Started.");
	LoggingEnabled(true);	
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Writes the header to the logfile.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LogStart(void)
{
	LogEntry("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n");
	LogEntry("<log>\n");
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Writes the footer for the log XML.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LogEnd(void)
{
	LogEntry("</log>\n");
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Destructor. Close the file and cleanup.
 */
/*-----------------------------------------------------------------------------------------------*/
CLogger::~CLogger()
{
	fclose(m_hLogFile);
	delete m_pcLogFileName;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Enable or disable logging for this instance.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LoggingEnabled(bool _bLog)
{
	m_bLoggingEnabled = _bLog;
	if(_bLog)
		LogEntry("<message text=\"Logging Enabled.\" />\n");
	else
		LogEntry("<message text=\"Logging Disabled.\" />\n");
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Writes a log entry to the file, copies format of printf.
Called by LogMessage(), LogError() and LogFunction().
 
\param		pcFormat,... Standard printf() style arguments.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LogEntry(const char *pcFormat, ...)
{	
	if(m_hLogFile==0)
		return;

	int iLen;

	va_list argList;

	va_start(argList, pcFormat);
	iLen = vsprintf(m_acTemp, pcFormat, argList);
	va_end(argList);

	fprintf(m_hLogFile, m_acTemp);
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Writes a message log entry to the file, copies format of printf.
 
\param		pcFormat,... Standard printf() style arguments.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LogMessage(const char *pcFormat, ...)
{
	if(!m_bLoggingEnabled)
		return;

	int iLen;

	va_list argList;
	char acTemp[256];

	va_start(argList, pcFormat);
	iLen = vsprintf(acTemp, pcFormat, argList);
	va_end(argList);

	LogEntry("<message text=\"%s\" />\n", acTemp);
}



/*-----------------------------------------------------------------------------------------------*/
/*!
Writes a function log entry to the file, copies format of printf.
 
\param		pcFormat,... Standard printf() style arguments.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LogFunction(const char *pcFormat, ...)
{
	if(!m_bLoggingEnabled)
		return;

	int iLen;

	va_list argList;
	char acTemp[256];

	va_start(argList, pcFormat);
	iLen = vsprintf(acTemp, pcFormat, argList);
	va_end(argList);

	LogEntry("<function name=\"%s\" />\n", acTemp);
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Writes a error log entry to the file, copies format of printf.
 
\param		pcFormat,... Standard printf() style arguments.
 */
/*-----------------------------------------------------------------------------------------------*/
void CLogger::LogError(const char *pcFormat, ...)
{
	if(!m_bLoggingEnabled)
		return;

	int iLen;

	va_list argList;
	char acTemp[256];

	va_start(argList, pcFormat);
	iLen = vsprintf(acTemp, pcFormat, argList);
	va_end(argList);

	LogEntry("<error text=\"%s\" />\n", acTemp);
}

