/*!
CLogger.h
 Menu Disc Demo
 
 Author: Jeroen Verbeek
 Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
 
 This software is provided without any guarantee.
 No responsibility taken for any problems caused by the use if this software.
 
 Anyone is authorised to use this program code and methods without warranty,
 all that is required is that you contact me (jeroen@happymakinggames.com),
 and tell me that you use it and what you use it in -
 all I want is to know is who uses it and where.
 
 I will not ask for payment in any way: this code is free.
 If you encounter a bug with the code, please tell me about it.
 If you have a bug-fix for the code please provide me with it.
 If you think of an improvement please tell me about it.
 You are not required to provide me with bug reports, bug fixes or improvements.
 If you require me to perform any work related to this code or
 not related to this code, please contact me: I am always open to offers.
 
 Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
 specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
 I am a big fan of circular menus (disc menus).
 I am of the opinion that they provide the quickest method of selecting an action
 from a structured tree of actions. A menu of actions organised following this method
 will, in time, allow almost "blind selection" of actions. Each action selection
 requires the same distance of mouse movement, just in a slightly diferent angle.
 In my opinion this seems to work best with menu discs of no more than 8 items.
 There is plenty of research to be found through the internet regarding the 
 advantages of circular menus, for example: muscle memory.
 Some websites I recommend you look at are:
 
 http://catalog.com/hopkins/piemenus/piemenus-hopkins.html
 
 http://www.piemenus.com/
 
 http://www.sm.luth.se/~david/carl/www/argonaut.html
 
 Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
 */
/*-----------------------------------------------------------------------------------------------*/
#ifndef _CLOGGER_H_
#define _CLOGGER_H_


/*-----------------------------------------------------------------------------------------------*/
/*!
\brief	Tiny logging class.
 
This can be used from anywhere to either write XML to a global log file (see: g_pGlobalLog), or
 use a local instantiation for (for example) specific object/class logs (or whatever, really!).
 This is obviously not a singleton.
 */
/*-----------------------------------------------------------------------------------------------*/
class CLogger
{
public:
	CLogger(char* _pcLogFileName);
	~CLogger();

	void LogStart(void);
	void LogEnd(void);

	inline char* LogFileName(void) const {return m_pcLogFileName;}
	inline FILE* FileHandle(void) const {return m_hLogFile;}
	inline bool LoggingEnabled(void) const {return m_bLoggingEnabled;}

	void LoggingEnabled(bool _bLog);
	void LogMessage(const char *_pcFormat, ...);
	void LogError(const char *_pcFormat, ...);
	void LogFunction(const char *_pcFormat, ...);

private:
	void LogEntry(const char *_pcFormat, ...);

private:

	FILE* m_hLogFile;
	char* m_pcLogFileName;
	char m_acTemp[512];
	bool m_bLoggingEnabled;
};


/*-----------------------------------------------------------------------------------------------*/
/*!
Global log object declaration.
 */
/*-----------------------------------------------------------------------------------------------*/
extern CLogger* g_pGlobalLog;


/*-----------------------------------------------------------------------------------------------*/
/*!
Global function checking declaration.
 */
/*-----------------------------------------------------------------------------------------------*/
extern bool CLoggerGlobalLogExists(void);


#endif
